@@ -27,6 +27,7 @@ from gi.repository import Gdk, GdkX11
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
+import uaclient.api.u.pro.packages.updates.v1 as ua
 
 GdkX11  # pyflakes
 
@@ -36,11 +37,11 @@ warnings.filterwarnings("ignore", "Accessed deprecated property",
 
 import apt_pkg
 import distro_info
-import json
 import os
 import subprocess
 import sys
 import time
+import threading
 from gettext import gettext as _
 
 import dbus
@@ -249,41 +250,42 @@ class UpdateManager(Gtk.Window):
                                          cancelled_update, error_occurred)
         self._start_pane(pane)
 
+    def _fetch_ua_updates(self):
+        self.ua_updates = ua.updates().updates
+
     def _get_ua_security_status(self):
         self.ua_security_packages = []
-        try:
-            p = subprocess.Popen(['pro', 'security-status', '--format=json'],
-                                 stdout=subprocess.PIPE)
-        except OSError:
-            pass
-        else:
-            while p.poll() is None:
-                while Gtk.events_pending():
-                    Gtk.main_iteration()
-                time.sleep(0.05)
-            s = json.load(p.stdout)
-            for package in s.get('packages', []):
-                if package.get('service_name', '') == 'standard-security':
-                    continue
-                status = package.get('status', '')
-                if (
-                    status == 'pending_attach'
-                    or status == 'pending_enable'
-                    or status == 'upgrade_available'
-                ):
-                    name = package.get('package', '')
-                    version = package.get('version', '')
-                    size = package.get('download_size', 0)
-                    downloadable = status == 'upgrade_available'
-                    self.ua_security_packages.append(
-                        (name, version, size, downloadable)
-                    )
-            self.cache.create_pro_cache(self.ua_security_packages)
+        t = threading.Thread(target=self._fetch_ua_updates, daemon=True)
+        t.start()
+        while t.is_alive():
+            while Gtk.events_pending():
+                Gtk.main_iteration()
+            time.sleep(0.05)
+        for package in self.ua_updates:
+            if (
+                package.provided_by == 'standard-security'
+                or package.provided_by == 'standard-updates'
+            ):
+                continue
+            status = package.status
+            if (
+                status == 'pending_attach'
+                or status == 'pending_enable'
+                or status == 'upgrade_available'
+            ):
+                name = package.package
+                version = package.version
+                size = package.download_size
+                downloadable = status == 'upgrade_available'
+                self.ua_security_packages.append(
+                    (name, version, size, downloadable)
+                )
+        self.cache.create_pro_cache(self.ua_security_packages)
 
     def _make_available_pane(self, install_count, need_reboot=False,
                              cancelled_update=False, error_occurred=False):
         self._check_hwe_support_status()
-        if install_count != 0 or len(self.ua_security_packages) > 0:
+        if install_count != 0:
             header = None
             desc = None
             if error_occurred:
